dnl **********************************************************************
dnl
dnl Copyright (c) 2008
dnl UChicago, LLC, operator of Argonne National Laboratory
dnl
dnl All Rights Reserved
dnl
dnl Author: Boyana Norris  norris@mcs.anl.gov
dnl **********************************************************************

dnl Check the PostgreSQL installation


AC_DEFUN([POSTGRESQL_CHECKS], [

	AC_SUBST(POSTGRESQL_ROOT)
	AC_SUBST(POSTGRESQL_LIB_DIR)
	AC_SUBST(POSTGRESQL_INCLUDE_DIR)

AC_REQUIRE([COMMON_VARS])

AC_ARG_WITH(postgresql,
    [  --with-postgresql=POSTGRESQL_ROOT     
                          the full path to the PostgreSQL installation.],
    , [with_postgresql=yes] )
AC_ARG_WITH(postgresql-inc,
    [  --with-postgresql-inc=POSTGRESQL_INC_DIR	 
                          the full path containing PostgreSQL include files.],
    , [with_postgresql_inc=no] )
AC_ARG_WITH(postgresql-lib,
    [  --with-postgresql-lib=POSTGRESQL_LIB_DIR	 
                          the full path containing PostgreSQL libraries.],
    , [with_postgresql_lib=no] )

case $with_postgresql in
    no)
	POSTGRESQL_ROOT=
	AC_MSG_ERROR([Postgresql is required for building the database library. $postgresql_err])
	;;
    yes)
	with_postgresql=_searching
	;;
    *)
  	POSTGRESQL_ROOT="$with_postgresql"	
	;;
esac
case $with_postgresql_inc in
    no)
	with_postgresql_inc=_searching
	;;
    yes)
	with_postgresql_inc=_searching
	;;
    *)
	POSTGRESQL_INCLUDE_DIR="$with_postgresql_inc"
	;;
esac
case $with_postgresql_lib in
    no)
	with_postgresql_lib=_searching
	;;
    yes)
	with_postgresql_lib=_searching
	;;
    *)
	POSTGRESQL_LIB_DIR="$with_postgresql_lib"
	;;
esac


postgresql_names="postgresql pgsql"
postgresql_versions="80 81 82 83"
postgresql_err="
PostgreSQL is required for building the database library. Try using 
one or more of the following options: 
    --with-postgresql, --with-postgresql-inc, or --with-postgresql-lib."
header="libpq-fe.h"
pglibs="pq"
pgincdirs=
pglibdirs=

dnl -------------------------------------------------
dnl Look for the libpq_fe.h header
dnl 
libpq_fe_h_ok="no"
if test ! -z "$POSTGRESQL_INCLUDE_DIR" -a -d "$POSTGRESQL_INCLUDE_DIR" ; then
	pgincdirs="$POSTGRESQL_INCLUDE_DIR"
elif test ! -z "$with_postgresql" -a -d "$with_postgresql" ; then
	pgincdirs="$with_postgresql $with_postgresql/include"
else
	pgincdirs="$include_dirs"
	for d in $include_dirs $soft_dirs ; do 
		for n in $postgresql_names ; do
			pgincdirs="$pgincdirs $d/$n $d/$n/include"
			for v in $postgresql_versions ; do
				pgincdirs="$pgincdirs $d/$n$v $d/$n$v/include"
			done
		done
	done
fi
AC_MSG_CHECKING([for $header])
for dir in $pgincdirs ; do 
	if test -f "$dir/$header" ; then 
		POSTGRESQL_INCLUDE_DIR=$dir
		libpq_fe_h_ok="yes"
		break
	fi
done
if test "libpq_fe_h_ok" = "no" ; then 
	AC_MSG_ERROR([Could not find postgresql header file: $header. $postgresql_err])
else
	AC_MSG_RESULT([found $POSTGRESQL_INCLUDE_DIR/$header])
fi
dnl AC_CHECK_HEADER([libpq-fe.h],AC_DEFINE(HAVE_LIBPQ_FE_H),[],[$POSTGRESQL_INCLUDE_DIR])
dnl AC_CHECK_HEADER([$POSTGRESQL_INCLUDE_DIR/postgres.h],AC_DEFINE(HAVE_POSTGRES_H))
dnl AC_DEFINE([HAVE_LIBPQ_FE_H], [], [libpg-fe.h postgresql header])
dnl AC_DEFINE([HAVE_POSTGRES_H], [], [postgres.h postgresql header])

dnl -------------------------------------------------
dnl Look for the libpq shared/dynamic and static libs
dnl
libpq_ok="no"
if test ! -z "$POSTGRESQL_LIB_DIR" -a -d "$POSTGRESQL_LIB_DIR" ; then
        pglibdirs="$POSTGRESQL_LIB_DIR"
elif test ! -z "$with_postgresql" -a -d "$with_postgresql" ; then
        pglibdirs="$with_postgresql $with_postgresql/lib"
else
	pglibdirs="$lib_dirs"
        for d in $lib_dirs $soft_dirs; do
                for n in $postgresql_names ; do
                        pglibdirs="$pglibdirs $d/$n $d/$n/lib"
                        for v in $postgresql_versions ; do
                                pglibdirs="$pglibdirs $d/$n$v $d/$n$v/lib"
                        done
                done
        done
fi
AC_MSG_CHECKING([for $pglib])
all_lib_ext="a $SHLIB_EXT_LIST"
for dir in $pglibdirs ; do
	for l in $pglibs ; do 
		for ext in $all_lib_ext ; do
        		if test -f "$dir/lib$l.$ext" ; then
				libpq_ok="yes"
                		POSTGRESQL_LIB_DIR=$dir
				break 3
			fi
		done
        done
done
if test "$libpq_ok" != "yes" ; then 
	AC_MSG_ERROR([Could not find postgresql libraries: $pglibs. $postgresql_err])
else
	AC_MSG_RESULT([found pq lib in $dir])
fi

])
