#include "Metadata.hxx"
#include "Comparator.hxx"
#include <iostream>

namespace cqosdb {

	string crossJoinStr(ParameterSet& set1, ParameterSet& set2){
		string str = "", tb1Name, tb2Name, fieldName1, fieldName2;
		int paramCount1, paramCount2;
		tb1Name = set1.getTBName();
		tb2Name = set2.getTBName();
		
		paramCount1 = set1.getNumOfParameters();
		paramCount2 = set2.getNumOfParameters();
		
		for (int ii=0;ii<paramCount1; ii++){
			fieldName1 = set1.getParameterAt(ii)->getName();
			for (int jj=0; jj<paramCount2; jj++){
				fieldName2 = set2.getParameterAt(jj)->getName();
				if (!fieldName1.compare(fieldName2)){
						str += tb1Name + "." + fieldName1 + " = " + tb2Name + "." + fieldName2 + " AND ";
				}
			}
		}
		
		if (str.size()>0)
			str.erase(str.size()-4, 4);
		return str;
	}

	void normString(string& type){
		Utility::trim(type);
		Utility::stringToLower((char *)type.c_str(),type.size());
	}
	
	DefaultProperty::DefaultProperty(string name, string type, int value){
		setName(name);
		setTBName("metadata_parameters");
		setValue(value);
		normString(type);
		setParamType(type);
		trialID = -1;
	}
	
	DefaultProperty::DefaultProperty(string name, string type, long value){
		setName(name);
		setTBName("metadata_parameters");
		setValue(value);
		normString(type);
		setParamType(type);
		trialID = -1;
		
	}
	
	DefaultProperty::DefaultProperty(string name, string type, double value){
		setName(name);
		setTBName("metadata_parameters");
		setValue(value);
		normString(type);
		setParamType(type);
		trialID = -1;
		
	}
	
	DefaultProperty::DefaultProperty(string name, string type, string value){
		setName(name);
		setTBName("metadata_parameters");
		setValue(value);
		normString(type);
		setParamType(type);
		trialID = -1;
		
	}
	
	DefaultProperty::DefaultProperty(string name, string type, bool value){
		setName(name);
		setTBName("metadata_parameters");
		setValue(value);
		normString(type);
		setParamType(type);
		trialID = -1;
		
	}
	
	DefaultProperty::DefaultProperty(string name, string type){
		setName(name);
		setTBName("metadata_parameters");
		normString(type);
		setParamType(type);
		hasAValue = false;
		trialID = -1;
		
	}
	
	DefaultProperty::DefaultProperty(){
		setTBName("metadata_parameters");
		trialID = -1;
		hasAValue = false;
	}
		
	string DefaultProperty::getClassName(){
		return string("DefaultProperty");
	}
	
	void DefaultProperty::setParamType(string type){
		propertyType = type;
	}
	
	string DefaultProperty::getParamType(){
		return propertyType;
	}
	
	void DefaultProperty::setName(string name){
		propertyName = name;
	}
	
	void DefaultProperty::setValue(int value){
		doubleValue = value;
		if (propertyType.empty())
			setParamType("int");
		hasAValue = true;
	}
	
	void DefaultProperty::setValue(long value){
		doubleValue = value;
		if (propertyType.empty())
			setParamType("long");
		hasAValue = true;
	}
	
	void DefaultProperty::setValue(double value){
		doubleValue = value;
		if (propertyType.empty())
			setParamType("double");
		hasAValue = true;
	}
	
	void DefaultProperty::setValue(string value){
		stringValue = value;
		if (propertyType.empty())
			setParamType("string");
		hasAValue = true;
	}
	
	void DefaultProperty::setValue(bool value){
		boolValue = value;
		if (propertyType.empty())
			setParamType("bool");
		hasAValue = true;
	}
	
	void DefaultProperty::setTBName(string tbName){
		tableName=tbName;
	}
	
	void DefaultProperty::setTrialID(int tID){
		trialID = tID;
	}
	
	void DefaultProperty::setInterEvt(string aName){
		interEvt = aName;
	}
		
			
	void DefaultProperty::setCatName(string cName){
		catName = cName;
	}
	
	string DefaultProperty::getCatName(){
		if (catName.empty()){
			fprintf(stderr, "Warning: the category name is not set yet\n");
		}

		return catName;
	}
	
	string DefaultProperty::getName(){
		if (propertyName.empty()){
			fprintf(stderr, "ERROR: the property is not set name yet\n");
			exit(-1);
		}

		return propertyName;
	}
	
	void DefaultProperty::getValue(int *value){
		if (propertyType.compare("int")==0)
			*value = (int) doubleValue;
		else
			fprintf(stderr, "ERROR: the property value is not of type INT");
	}
	
	void DefaultProperty::getValue(long *value){
		if (propertyType.compare("long")==0)
			*value = (long) doubleValue;
		else if (propertyType.compare("int")==0)
			*value = (long) doubleValue;
		else
			fprintf(stderr, "ERROR: the property value is not of type LONG");
	}

	void DefaultProperty::getValue(double *value){
		if ((propertyType.compare("int")==0) || (propertyType.compare("long")==0) || (propertyType.compare("double")==0))
			*value = doubleValue;
		else
			fprintf(stderr, "ERROR: the property value is not of type DOUBLE");
	}

	void DefaultProperty::getValue(string *value){
		if (propertyType.compare("string")==0)
			*value = stringValue;
		else
			fprintf(stderr, "ERROR: the property value is not of type string");
	}
	
	void DefaultProperty::getValue(bool *value){
		if (propertyType.compare("bool")==0)
			*value = boolValue;
		else
			fprintf(stderr, "ERROR: the property value is not of type boolean");
	}

	string DefaultProperty::getTBName(){
		if (tableName.empty()){
			fprintf(stderr, "ERROR: the matrix property is not set table name yet\n");
		}
			
		return tableName;
	}
		
	int DefaultProperty::getTrialID(){
		return trialID;
	}
	
	string DefaultProperty::getInterEvt(){
		return interEvt;
	}
	
	bool DefaultProperty::compareWith(Parameter& another, double tolerance, int relation){
	
		if (propertyType.compare("string")==0){
			string anotherV;
			another.getValue(&anotherV);
			
			switch(relation){
			case LT :	
					return (stringValue.compare(anotherV) < 0);
			case LTEQ :
					return (stringValue.compare(anotherV) <= 0);
			case EQ	:
					return (stringValue.compare(anotherV) == 0);
			case NEQ :
					return (stringValue.compare(anotherV) != 0);							
			case GT:
					return (stringValue.compare(anotherV) > 0);	
			case GTEQ :
					return (stringValue.compare(anotherV) >= 0);
			default:
					fprintf(stderr,"ERROR: unknown relation!\n");
					fflush(stderr);
					return false;
			}
		}
		
		if (propertyType.compare("bool")==0){
			bool anotherV;
			another.getValue(&anotherV);
			
			switch(relation){
				case EQ:
					return (anotherV == boolValue);
				case NEQ:
					return (anotherV != boolValue);
				default:
					fprintf(stderr,"ERROR: unknown relation!\n");
					fflush(stderr);
					return false;
			}
		}
		
		double thisValue, anotherValue;
		getValue(&thisValue);
		another.getValue(&anotherValue);
		
		switch(relation){
			case LT :	
					return ( (thisValue - anotherValue) < tolerance);
			case LTEQ :
					return ( (thisValue - anotherValue) <= tolerance);
			case EQ	:
					return ( (thisValue - anotherValue) == tolerance);
			case NEQ :
					return ( (thisValue - anotherValue) != tolerance);							
			case GT:
					return ( (thisValue - anotherValue) > tolerance);	
			case GTEQ :
					return ((thisValue - anotherValue) >= tolerance);
			case BT:
					return ((thisValue - anotherValue) <= tolerance) && ((anotherValue - thisValue) <= tolerance) ;
			default:
					fprintf(stderr,"ERROR: unknown relation!\n");
					fflush(stderr);
					return false;
		}
		
	}
	
	
		bool DefaultProperty::compareWith(Parameter& another, int relation){
	
		if (propertyType.compare("string")==0){
			string anotherV;
			another.getValue(&anotherV);
			
			switch(relation){
			case LT :	
					return (stringValue.compare(anotherV) < 0);
			case LTEQ :
					return (stringValue.compare(anotherV) <= 0);
			case EQ	:
					return (stringValue.compare(anotherV) == 0);
			case NEQ :
					return (stringValue.compare(anotherV) != 0);							
			case GT:
					return (stringValue.compare(anotherV) > 0);	
			case GTEQ :
					return (stringValue.compare(anotherV) >= 0);
			default:
					fprintf(stderr,"ERROR: unknown relation!\n");
					fflush(stderr);
					return false;
			}
		}
		
		if (propertyType.compare("bool")==0){
			bool anotherV;
			another.getValue(&anotherV);
			
			switch(relation){
				case EQ:
					return (anotherV == boolValue);
				case NEQ:
					return (anotherV != boolValue);
				default:
					fprintf(stderr,"ERROR: unknown relation!\n");
					fflush(stderr);
					return false;
			}
		}
		
		double thisValue, anotherValue;
		getValue(&thisValue);
		another.getValue(&anotherValue);
		
		switch(relation){
			case LT :	
					return (thisValue < anotherValue);
			case LTEQ :
					return (thisValue <= anotherValue);
			case EQ	:
					return (thisValue == anotherValue);
			case NEQ :
					return (thisValue != anotherValue);							
			case GT:
					return (thisValue > anotherValue);	
			case GTEQ :
					return (thisValue >= anotherValue);
			default:
					fprintf(stderr,"ERROR: unknown relation!\n");
					fflush(stderr);
					return false;
		}
		
	}

	string DefaultProperty::getDBConditions(){
	
		string query=" ";
		
		query += tableName + ".parameter_name = \'";
		query += propertyName + "\' ";
		
		
		if (!catName.empty()){
			query += " AND " + tableName + ".category_name = \'";
			query += catName + "\' ";
		}
		
		if (!propertyType.empty()){
			query += " AND " + tableName + ".parameter_type = \'";
			query += propertyType + "\' ";
		}
		
		if (trialID >= 0)
			query += " AND " + tableName + ".trial=" + Utility::T_as_string<int>(trialID);
			
		if (!interEvt.empty())
			query += " AND " + tableName + ".interval_event= \'" + interEvt + "\' ";
		
		if (hasAValue){
			if ((propertyType.compare("int")==0) || (propertyType.compare("long")==0) || (propertyType.compare("double")==0) ){ 
				query += " AND " + tableName +".parameter_value = ";
				query += Utility::T_as_string<double>(doubleValue);
			}
			else if (propertyType.compare("string")==0){
				query += " AND " + tableName +".parameter_value = ";
				query += stringValue;
			}
			else if (propertyType.compare("bool")==0){
				query += " AND " + tableName +".parameter_value = ";
				query += Utility::T_as_string<bool>(boolValue);
			}			
		}
		
		return query;
	}
	
	string DefaultProperty::getDBConditions(int tbIndex, double value, int relation){
		string query=" ";
		string tbAlias=tableName;
		
		if (tbIndex >= 0)
			tbAlias += Utility::T_as_string<int>(tbIndex);
			
		query += tbAlias +".parameter_name = \'";
		query += propertyName + "\'";
		
		if (!catName.empty()){
			query += " AND " + tbAlias + ".category_name = \'";
			query += catName + "\' ";
		}
		
		if (!propertyType.empty()){
			query += " AND " + tbAlias + ".parameter_type = \'";
			query += propertyType + "\' ";
		}

		if (trialID >= 0)
			query += " AND " + tbAlias + ".trial=" + Utility::T_as_string<int>(trialID);
			
		if (!interEvt.empty())
			query += " AND " + tbAlias + ".interval_event=" + interEvt;

			
		query += " AND " + tbAlias +".parameter_value";
		
		if (hasAValue){
			if ((propertyType.compare("int")==0) || (propertyType.compare("long")==0) || (propertyType.compare("double")==0) ){ 
				
				switch(relation){
					case LT :	
							query += " < " + Utility::T_as_string<double>(value);
							break;
					case LTEQ :
							query += " <= " + Utility::T_as_string<double>(value);
							break;
					case EQ	:
							query += " = " + Utility::T_as_string<double>(value);
							break;
					case NEQ :
							query += " <> " + Utility::T_as_string<double>(value);	
							break;						
					case GT:
							query += " > " + Utility::T_as_string<double>(value);	
							break;
					case GTEQ :
							query += " >= " + Utility::T_as_string<double>(value);
							break;
					default:
							fprintf(stderr,"ERROR: unknown relation!\n");
							fflush(stderr);
							return "";
				}				
				return query;
			}
			else{
				fprintf(stderr,"ERROR: compare different data types!\n");
				fflush(stderr);
				return "";
			}
		}
	}

	string DefaultProperty::getDBConditions(int tbIndex, string aStr, int relation){
		string query=" ";
		string tbAlias=tableName;
		
		if (tbIndex >= 0)
			tbAlias += Utility::T_as_string<int>(tbIndex);
			
		query += tbAlias +".property_name = \'";
		query += propertyName + "\'";
		
		if (!catName.empty()){
			query += " AND " + tbAlias + ".category_name = \'";
			query += catName + "\' ";
		}
		
		if (!propertyType.empty()){
			query += " AND " + tbAlias + ".parameter_type = \'";
			query += propertyType + "\' ";
		}

		if (trialID >= 0)
			query += " AND " + tbAlias + ".trial=" + Utility::T_as_string<int>(trialID);
			
		if (!interEvt.empty())
			query += " AND " + tbAlias + ".interval_event=" + interEvt;

			
		query += " AND " + tbAlias +".parameter_value";
		
		if (propertyType.compare("string")==0){ // only compare if the two strings are the same.
						
			switch(relation){
			case EQ	:
					query += " = \'" + aStr + "\'";
					return query;
			case NEQ :
					query += " <> \'" + aStr + "\'";
					return query;							
			default:
					fprintf(stderr,"ERROR: unknown relation for string comparison!\n");
					fflush(stderr);
					return "";
			}
		}
		else {
			fprintf(stderr,"ERROR: compare different data types!\n");
			fflush(stderr);
			return "";
		}
	}

	string DefaultProperty::getDBConditions(int tbIndex, bool boolVal, int relation){
		string query=" ";
		string tbAlias=tableName;
		
		if (tbIndex >= 0)
			tbAlias += Utility::T_as_string<int>(tbIndex);
			
		query += tbAlias +".property_name = \'";
		query += propertyName + "\'";
		
		if (!catName.empty()){
			query += " AND " + tbAlias + ".category_name = \'";
			query += catName + "\' ";
		}
		
		if (!propertyType.empty()){
			query += " AND " + tbAlias + ".parameter_type = \'";
			query += propertyType + "\' ";
		}

		if (trialID >= 0)
			query += " AND " + tbAlias + ".trial=" + Utility::T_as_string<int>(trialID);
			
		if (!interEvt.empty())
			query += " AND " + tbAlias + ".interval_event=" + interEvt;

			
		query += " AND " + tbAlias +".parameter_value";
		
		if (propertyType.compare("bool")==0){ // only compare if the two bpols are the same.
						
			switch(relation){
			case EQ	:
					query += " = \'" + Utility::T_as_string<bool>(boolVal) + "\'";
					return query;
			case NEQ :
					query += " <> \'" + Utility::T_as_string<bool>(boolVal) + "\'";
					return query;							
			default:
					fprintf(stderr,"ERROR: unknown relation for boolean comparison!\n");
					fflush(stderr);
					return "";
			}
		}
		else {
			fprintf(stderr,"ERROR: compare different data types!\n");
			fflush(stderr);
			return "";
		}
	}
	
	/*** End of DefaultProperty ***/

	PropertySet::PropertySet(vector<DefaultProperty>& pVec){
		paramVec = pVec;
	}

	PropertySet::PropertySet(const PropertySet& other){
	
		for (int index=0; index < other.paramVec.size(); ++index){
				paramVec.push_back(other.paramVec[index]);			
		}
		
	}
	
	PropertySet::~PropertySet(){
		paramVec.clear();
	}
	
	string PropertySet::getClassName(){
		return string("PropertySet");
	}
	
	int PropertySet::getNumOfParameters(){
		return paramVec.size();
	}
	
	string PropertySet::getTBName(){
		if (paramVec.size()>0)
			return paramVec[0].getTBName();
			
		fprintf(stderr, "No parameter in the category yet.");
		return "";
	}
	
	void PropertySet::addAParameter(Parameter& aParam){
		string typeName("DefaultProperty");

                if (aParam.getClassName().compare(typeName) !=0){
                        fprintf(stderr,"ERROR: the parameter to be added is not of type DefaultProperty!\n");
                        fflush(stderr);
                        return;
                }
	
		DefaultProperty& dp = dynamic_cast<DefaultProperty&>(aParam);
		for (int index=0; index < paramVec.size(); ++index){
			if (paramVec[index].getName().compare(dp.getName())==0){
				fprintf(stderr,"ERROR: the parameter has already added to the set!\n");
				fflush(stderr);
				return;
			}
		}
		paramVec.push_back(dp);
	}		
			

	Parameter* PropertySet::getParameterAt(string name){
		for (int index=0; index < paramVec.size(); ++index){
			if (paramVec[index].getName().compare(name)==0){
				return &paramVec[index];
			}
		}
		
		fprintf(stderr,"ERROR: no such parameter found in the category!\n");
		return NULL;		
	}
	
	Parameter* PropertySet::getParameterAt(int index){
		
		if ((index < 0) || (index >= paramVec.size())){
			fprintf(stderr,"ERROR: index beyond scope of the category!\n");
			return NULL;
		}
		
		return &paramVec[index];
	}
	
	bool PropertySet::compareWith(ParameterSet& another, map<string, double> tMap, map<string, int> rMap){
		
		if (another.getClassName().compare(getClassName())!=0){
			fprintf(stderr,"ERROR: can not compare different ParameterSet classes!\n");
			return false;
		}
		
		PropertySet& castA = dynamic_cast<PropertySet&>(another);
		PropertySetComparator myComparator(*this, castA, tMap, rMap);
		
		return myComparator.doCompare();
	}

	bool PropertySet::compareWith(ParameterSet& another, map<string, int> rMap){
		
		if (another.getClassName().compare(getClassName())!=0){
			fprintf(stderr,"ERROR: can not compare different ParameterSet classes!\n");
			return false;
		}
		
		PropertySet& castA = dynamic_cast<PropertySet&>(another);
		PropertySetComparator myComparator(*this, castA, rMap);
		
		return myComparator.doCompare();
	}
			
	string PropertySet::getDBConditions(int* tbIndices, map<string, double> tMap, map<string, int> rMap){
	
		string conditions=" ", propertyName;
		int size = getNumOfParameters();
		double tolerance; int relation;
		
		for (int index =0; index<size; index++){
			
			if (index != 0)
				conditions += " AND ";
			
			Parameter* aParam = dynamic_cast<Parameter*>(getParameterAt(index));
			// TODO check if cast succeeded
			propertyName = aParam->getName();
			tolerance = tMap[propertyName];
			relation = rMap[propertyName];
			
			conditions += aParam->getDBConditions(tbIndices[index], tolerance, relation);
		}
		
		return conditions;	
	}
	
	PropertySet& PropertySet::operator=(const PropertySet& other){
		if (&other != this){
			paramVec.clear();
			for (int index=0; index < other.paramVec.size(); ++index){
				paramVec.push_back(other.paramVec[index]);			
			}
		}
		
		return *this;
	}
	
	ParameterSet* PropertySet::clone(){
		PropertySet* aSet = new PropertySet;
		(*aSet) = *this;
		return aSet;
	}
	
	/************************ ENd of PropertySet **************************/
	
}
