#include "ParseDBConfig.hxx"

namespace cqosdb {
	ConfigFile::ConfigFile( string filename, string delimiter,string comment)
		: myDelimiter(delimiter), myComment(comment)
	{
		ifstream in( filename.c_str() );
		
		if( !in ) throw file_not_found( filename ); 
		
		in >> (*this);
	}


	ConfigFile::ConfigFile()
		: myDelimiter( string(1,':') ), myComment( string(1,'#') )
	{
		// Construct a ConfigFile without a file; empty
	}


	void ConfigFile::remove( const string& key )
	{
		myContents.erase( myContents.find( key ) );
		return;
	}


	bool ConfigFile::keyExists( const string& key ) const
	{
		mapci p = myContents.find( key );
		return ( p != myContents.end() );
	}

	ostream& operator<<( ostream& os, const ConfigFile& cf )
	{
		for( ConfigFile::mapci p = cf.myContents.begin();
			 p != cf.myContents.end();
			 ++p )
		{
			os << p->first << " " << cf.myDelimiter << " ";
			os << p->second << endl;
		}
		return os;
	}


	istream& operator>>( istream& is, ConfigFile& cf )
	{
		
		typedef string::size_type pos;
		const string& delim  = cf.myDelimiter;  // separator
		const string& comm   = cf.myComment;    // comment
		const pos skip = delim.length();        // length of separator
		
		string nextline = "";  // might need to read ahead to see where value ends
		
		while( is || nextline.length() > 0 )
		{
			// Read an entire line at a time
			string line;
			if( nextline.length() > 0 )
			{
				line = nextline;  // we read ahead; use it now
				nextline = "";
			}
			else
			{
				getline( is, line );
			}
			
			// Ignore comments
			line = line.substr( 0, line.find(comm) );
			
			
			// Parse the line if it contains a delimiter
			pos delimPos = line.find( delim );
			if( delimPos < string::npos )
			{
				// Extract the key
				string key = line.substr( 0, delimPos );
				line.replace( 0, delimPos+skip, "" );
				
				// See if value continues on the next line
				// Stop at blank line, next line with a key, end of stream,
				// or end of file sentry
				bool terminate = false;
				while( !terminate && is )
				{
					getline( is, nextline );
					terminate = true;
					
					string nlcopy = nextline;
					Utility::trim(nlcopy);
					if( nlcopy == "" ) continue;
					
					nextline = nextline.substr( 0, nextline.find(comm) );
					if( nextline.find(delim) != string::npos )
						continue;
					
					nlcopy = nextline;
					Utility::trim(nlcopy);
					if( nlcopy != "" ) line += "\n";
					line += nextline;
					terminate = false;
				}
				
				// Store key and value
				Utility::trim(key);
				Utility::trim(line);
				cf.myContents[key] = line;  // overwrites if key is repeated
			}
		}
		
		return is;
	}


}
